#include <FastLED.h>

#define trigPin       10  // ultrasonic sensro trig ping
#define echoPin       11  // ultrasonic sensro echo ping
#define NUM_LEDS      60  // How many leds in your strip?
#define DATA_PIN      3   // data pin for LED strip
#define ZONE          30  // default zone distance is 30 cms / a foot
#define MAXRR         30  // default distance for Flashing Red is 30 cms / a foot
#define MAXR          MAXRR+ZONE  // distance for Red
#define MAXY          MAXR+ZONE   // distance for Yellow/Amber
#define MAXG          MAXY+ZONE   // distance for Green
#define MAXFLASHCOUNT 30  // max times to flash

CRGB leds[NUM_LEDS];        // Define the array of leds

float duration, distance;
int flashcount, LEDpercm;

void setup() 
{
  // initialize the LED strip
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS);  // GRB ordering is typical
  FastLED.clear();  // clear all pixel data
  FastLED.show();

  // set the trig and echo pin modes
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);

  LEDpercm = NUM_LEDS / ZONE ; // number of LEDs to glow per cm of distance measured

}

void loop() 
{
  CallSensor();
}

void CallSensor()
{
  // initiate ultrasonic sensor to fire
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  duration = pulseIn(echoPin, HIGH);
  distance = (duration / 2) * 0.0344; // use speed of sound to calc rountrip distance and divide by 2 to get one way distance

  delay(200); // give a delay between successive firings of ultrasonic sensor

  if (distance > MAXG) // if distance is greater than MAXG clear all LEDs
  {
    fill_solid( leds, NUM_LEDS, CRGB::Black); 
    FastLED.show();
    return;
  }
  else if (distance > MAXY && distance < MAXG) 
    ChangePixelColors(MAXG, distance, 0, 255, 0); // green color
  else if (distance > MAXR && distance < MAXY) 
    ChangePixelColors(MAXY, distance, 255, 126, 0); //Amber color
  else if (distance > MAXRR && distance < MAXR) 
    ChangePixelColors (MAXR, distance, 255, 0, 0); //red color
  else if (distance <= MAXRR) // we are in SOS zone, show flashing red
    ShowFlashingRed();
}

void ShowFlashingRed()
{
  if (flashcount >= MAXFLASHCOUNT) //stop flashing after MAX times
  {
    fill_solid( leds, NUM_LEDS, CRGB::Black);
    FastLED.show();
    return;
  }

  flashcount++; // increment the times we have flashed red

  fill_solid( leds, NUM_LEDS, CRGB(255, 0, 0)); //fill full LED strip with red color
  FastLED.show();
  delay (250);
  fill_solid( leds, NUM_LEDS, CRGB::Black);
  FastLED.show();
  delay (250);
}

void ChangePixelColors(int zonemax, float distance, int R, int G, int B)
{
  int countLED;
  fill_solid(leds, NUM_LEDS, CRGB::Black); // reset the LED strip to black (or blank)

  // calculate the difference of distance between zone max and object distance. This is the distance
  // the object is at within the zone. E.g if zonemax is 120cm and object is at 100cm, we get 20cm
  // Use this 20cm and LEDpercm to calculate how many LEDs to show
  countLED = round((zonemax - distance)* LEDpercm) ; 
   
  for (int n = 0; n <= countLED; n++) // fill the LEDs with right color and until countLED 
    leds[n] = CRGB (R, G, B);
  
  FastLED.show();
  flashcount = 0; // we are no longer flashing red so reset this to zero
}

